<?php

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

class FV_Player_Subtitles {

  /**
   * List of RTL languages
   * @see https://meta.wikimedia.org/wiki/Template:List_of_language_names_ordered_by_code
   *
   * @var array
   */
  public $aRtlSubtitles = array(
    'ar', 'arc', 'arz', 'ckb', 'dv', 'fa', 'ha', 'he', 'khw', 'ks', 'ku', 'ps', 'sd', 'uz_af', 'yi'
  );

  public function __construct() {
    add_filter('fv_player_item', array($this, 'add_subtitles'), 10, 3 );
  }

  function add_subtitles( $aItem, $index ) {
    global $fv_fp;

    $aSubtitles = $fv_fp->get_subtitles($index);
    if( count($aSubtitles) == 0 ) return $aItem;

    $aLangs = flowplayer::get_languages();
    $countSubtitles = 0;
    $aOutput = array();

    foreach( $aSubtitles AS $key => $subtitles ) {
      if( $key == 'iw' ) $key = 'he';
      if( $key == 'in' ) $key = 'id';
      if( $key == 'jw' ) $key = 'jv';
      if( $key == 'mo' ) $key = 'ro';
      if( $key == 'sh' ) $key = 'sr';

      $objSubtitle = new stdClass;
      if( $key == 'subtitles' ) {
        $aLang = explode('-', get_bloginfo('language'));
        if( !empty($aLang[0]) ) $objSubtitle->srclang = $aLang[0];
        $sCode = $aLang[0];

        $sCaption = '';
        if( !empty($sCode) && $sCode == 'en' ) {
          $sCaption = 'English';

        } else if( !empty($sCode) ) {
          $translations = get_site_transient( 'available_translations' );
          $sLangCode = str_replace( '-', '_', get_bloginfo('language') );
          if( $translations && isset($translations[$sLangCode]) && !empty($translations[$sLangCode]['native_name']) ) {
            $sCaption = $translations[$sLangCode]['native_name'];
          }

        }

        if( $sCaption ) {
          $objSubtitle->label = $sCaption;
        }

      } else {
        $objSubtitle->srclang = $key;
        $objSubtitle->label = $aLangs[strtoupper($key)];
      }

      if( in_array( strtolower($key), $this->aRtlSubtitles) ) {
        $objSubtitle->rtl = true;
      }

      $objSubtitle->src = $subtitles;
      // default subtitle
      if( $countSubtitles == 0 && $fv_fp->_get_option('subtitleOn') ) {
        $objSubtitle->fv_default = true;
      }

      $aOutput[] = $objSubtitle;

      $countSubtitles++;
    }

    if( count($aSubtitles) ) {
      $aItem['subtitles'] = $aOutput;
    }

    return $aItem;
  }

}

$FV_Player_Subtitles = new FV_Player_Subtitles();
